/* SISTEMAS INTEGRADOS TECNISA
   PLANEJAMENTO ALIMENTAO INDUSTRIAL

NOME: VERIFICA_CUSTO.SQL

DESCRIO:
STORED PROCEDURE AUXILIAR UTILIZADA NA FUNO CUSTO_DO_PRODUTO.
FAZ OS CLCULOS E RETORNA OS CUSTOS.

P_CDFILIAL     - CDIGO DA FILIAL.
P_CDPRODUTO    - CDIGO DO PRODUTO.
P_NRTIPOCUST   - NMERO DO TIPO DE CUSTO (1 A 10)
P_DTREFERENCIA - DATA DO CUSTO.
P_VRULTICUST   - RECEBE O VALOR CALCULADO PARA LTIMO CUSTO DE COMPRAS.
P_VRULTICOTA   - RECEBE O VALOR CALCULADO PARA LTIMO CUSTO DE COTAO.
P_VRCUSTMEDI   - RECEBE O VALOR CALCULADO PARA CUSTO MDIO.

TIPOS DE CUSTO:

01 - LTIMO CUSTO DE COMPRA
02 - ULTIMO CUSTO ATUALIZADO
03 - LTIMO CUSTO (EM MOEDA FORTE)
04 - ULTIMO CUSTO DE COTAO (AF)
05 - CUSTO MDIO (LQUIDO) ATUAL
06 - CUSTO MDIO (LQUIDO) DA POCA
07 - CUSTO MDIO (LQUIDO EM MOEDA FORTE)
08 - CUSTO MDIO (BRUTO) ATUAL
09 - CUSTO MDIO (BRUTO) DA POCA
10 - CUSTO MDIO (BRUTO EM MOEDA FORTE)

CRIAO:
12/08/98 - EDERSON

MANUTENES:
18/08/99 - ROBERTO - PARMETROS DA PROCEDURE POSICAO_ESTOQUE.
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'VERIFICA_CUSTO' AND
      	        TYPE = 'P' 			       AND
	              U.UID = O.UID             AND
      	        U.NAME = USER)
DROP PROCEDURE VERIFICA_CUSTO
GO
CREATE PROCEDURE VERIFICA_CUSTO @P_CDFILIAL     VARCHAR(04),
                                @P_CDPRODUTO    VARCHAR(10),
                                @P_NRTIPOCUST   NUMERIC,
                                @P_DTREFERENCIA DATETIME,
                                @P_VRULTICUST   NUMERIC(15,6) OUTPUT,
                                @P_VRULTICOTA   NUMERIC(15,6) OUTPUT,
                                @P_VRCUSTMEDI   NUMERIC(15,6) OUTPUT,
                                @P_ALMOXARIFE   VARCHAR(06) = NULL,
                                @P_LOCALESTOQ   VARCHAR(06) = NULL,
                                @P_NRLOTEESTQ   VARCHAR(20) = NULL,
                                @P_NRSUBLOTE    VARCHAR(04) = NULL
AS
   DECLARE @V_DOLAREPOCA NUMERIC(15,6);
   DECLARE @V_DOLARATUAL NUMERIC(15,6);

   DECLARE @V_DTPOSICAO  DATETIME;
   DECLARE @V_DTPOSIAUXI DATETIME;

   DECLARE @V_QTDEESTO   NUMERIC(15,6);
   DECLARE @V_VALRESTO   NUMERIC(15,6);
   DECLARE @V_VALRBRUT   NUMERIC(15,6);

   DECLARE @V_DTPROCESSA DATETIME;

   DECLARE @V_VRULTCOMPR   NUMERIC(15,6);
   DECLARE @V_DTULTCOMPR   DATETIME;
   DECLARE @V_VRPRECUAFOFI NUMERIC;

   DECLARE @V_CDINDICEMF VARCHAR(04);

   DECLARE @V_DTREFE DATETIME;

   DECLARE @V_VRINDICE NUMERIC(15,6);

   DECLARE @V_CDFILCENTCUST VARCHAR(04);
   DECLARE @V_CDREGIAO      VARCHAR(03);
   DECLARE @V_VRLISTAPRECO  NUMERIC;

BEGIN

   DECLARE C_DATAPROC CURSOR FOR
   SELECT
      ISNULL(DTPROCESSA, GETDATE()) AS DTPROCESSA
   FROM
      PARAMFILIAL A, PARAMGERAL B
   WHERE
      (A.CDFILIAL = @P_CDFILIAL) AND
      (B.IDUTILDTPROC <> 'N');

   /* - VERIFICA A DATA DE PROCESSAMENTO DA FILIAL -------------------------- */

   OPEN C_DATAPROC;
   FETCH NEXT FROM C_DATAPROC INTO @V_DTPROCESSA;
   IF (@@FETCH_STATUS <> 0) BEGIN
      SELECT @V_DTPROCESSA = GETDATE()
      FROM PARAMGERAL;
   END;
   CLOSE C_DATAPROC;
   DEALLOCATE C_DATAPROC;

   /* ----------------------------------------------------------------------- */

   SET @P_VRULTICUST = 0;
   SET @P_VRULTICOTA = 0;
   SET @P_VRCUSTMEDI = 0;

   IF @P_NRTIPOCUST IN (1, 2, 3) BEGIN

      SET @P_VRULTICUST = 0;

      DECLARE C_CDINDICEMF CURSOR FOR
      SELECT
         CDINDICEMF
      FROM
         PARAMGERAL;
      OPEN  C_CDINDICEMF;
      FETCH NEXT FROM C_CDINDICEMF INTO @V_CDINDICEMF;
      IF (@@FETCH_STATUS <> 0) BEGIN
         SET @V_CDINDICEMF = ' ';
      END;
      CLOSE C_CDINDICEMF;
      DEALLOCATE C_CDINDICEMF;

      DECLARE C_FIND_PRODFILI CURSOR FOR
      SELECT
         VRULTCOMPR,
         DTULTCOMPR,
         VRPRECUAFOFI
      FROM
         PRODFILI
      WHERE
         (CDFILIAL  = @P_CDFILIAL) AND
         (CDPRODUTO = @P_CDPRODUTO);

      OPEN  C_FIND_PRODFILI;
      FETCH NEXT FROM C_FIND_PRODFILI INTO @V_VRULTCOMPR, @V_DTULTCOMPR,
                                           @V_VRPRECUAFOFI;
      IF (@@FETCH_STATUS <> 0) BEGIN
         SET @V_VRULTCOMPR   = 0;
         SET @V_DTULTCOMPR   = CONVERT(DATETIME, '01/01/0001');
         SET @V_VRPRECUAFOFI = 0;
      END;
      CLOSE C_FIND_PRODFILI;
      DEALLOCATE C_FIND_PRODFILI;

      IF @P_NRTIPOCUST IN (2, 3) BEGIN
         SET @V_DTREFE = @V_DTULTCOMPR;

         DECLARE C_FIND_MOVINDDI CURSOR FOR
         SELECT
            VRINDICE
         FROM
            MOVINDICE
         WHERE
            (CDTPINDICE = '1')    AND
            (DTREFIND = @V_DTREFE) AND
            (CDINDICE = @V_CDINDICEMF);

         OPEN  C_FIND_MOVINDDI;
         FETCH NEXT FROM C_FIND_MOVINDDI INTO @V_VRINDICE;
         IF (@@FETCH_STATUS <> 0) BEGIN
            SET @V_VRINDICE = 0;
         END;
         CLOSE C_FIND_MOVINDDI;

         SET @V_DOLAREPOCA = @V_VRINDICE;
         SET @V_DTREFE     = @V_DTPROCESSA;

         OPEN  C_FIND_MOVINDDI;
         FETCH NEXT FROM C_FIND_MOVINDDI INTO @V_VRINDICE;
         IF (@@FETCH_STATUS <> 0) BEGIN
            SET @V_VRINDICE = 0;
         END;
         CLOSE C_FIND_MOVINDDI;
         DEALLOCATE C_FIND_MOVINDDI;

         SET @V_DOLARATUAL = @V_VRINDICE;

         IF @V_DOLAREPOCA = 0 BEGIN
            SET @V_DOLAREPOCA = @V_DOLARATUAL;
         END;
      END;

      SET @P_VRULTICUST = 0;

      IF @P_NRTIPOCUST = 1 BEGIN
         SET @P_VRULTICUST = @V_VRULTCOMPR;
      END;
      IF @P_NRTIPOCUST = 2 BEGIN
         IF @V_DOLAREPOCA <> 0 BEGIN
            SET @P_VRULTICUST = (@V_VRULTCOMPR/@V_DOLAREPOCA) * @V_DOLARATUAL;
         END;
      END;
      IF @P_NRTIPOCUST = 3 BEGIN
         IF @V_DOLAREPOCA <> 0 BEGIN
            SET @P_VRULTICUST = @V_VRULTCOMPR/@V_DOLAREPOCA;
         END;
      END;
   END;

   IF @P_NRTIPOCUST = 4 BEGIN
      SET @P_VRULTICOTA = 0;

      DECLARE C_FIND_PRODFILI CURSOR FOR
      SELECT
         VRULTCOMPR,
         DTULTCOMPR,
         VRPRECUAFOFI
      FROM
         PRODFILI
      WHERE
         (CDFILIAL  = @P_CDFILIAL) AND
         (CDPRODUTO = @P_CDPRODUTO);

      OPEN C_FIND_PRODFILI;
      FETCH NEXT FROM C_FIND_PRODFILI INTO @V_VRULTCOMPR, @V_DTULTCOMPR,
                                           @V_VRPRECUAFOFI;
      IF (@@FETCH_STATUS <> 0) BEGIN
         SET @V_VRULTCOMPR   = 0;
         SET @V_DTULTCOMPR   = CONVERT(DATETIME, '01/01/0001');
         SET @V_VRPRECUAFOFI = 0;
      END;
      CLOSE C_FIND_PRODFILI;
      DEALLOCATE C_FIND_PRODFILI;

      SET @P_VRULTICOTA = @V_VRPRECUAFOFI;
   END;


   IF @P_NRTIPOCUST IN (5,6,7,8,9,10) BEGIN

      SET @P_VRCUSTMEDI = 0;

      IF @P_NRTIPOCUST IN (5,7,8,10) BEGIN
         SET @V_DTPOSICAO = @V_DTPROCESSA + 1;
      END;
      IF @P_NRTIPOCUST IN (6,9) BEGIN
         SET @V_DTPOSICAO = @P_DTREFERENCIA + 1;
      END;

      EXEC POSICAO_ESTOQUE @P_CDFILIAL,
                           @P_CDPRODUTO,
                           @V_DTPOSICAO,
                           'S',
                           @P_ALMOXARIFE,
                           @P_LOCALESTOQ,
                           @P_NRLOTEESTQ,
                           @P_NRSUBLOTE,
                           NULL,
                           NULL,
                           NULL,
                           @P_QTDEESTO = @V_QTDEESTO OUTPUT,
                           @P_VALRESTO = @V_VALRESTO OUTPUT,
                           @P_VALRBRUT = @V_VALRBRUT OUTPUT;

      /* LQUIDO */
      IF @P_NRTIPOCUST IN (5,6,7) BEGIN
         IF @V_QTDEESTO <> 0 BEGIN
            SET @P_VRCUSTMEDI = @V_VALRESTO / @V_QTDEESTO;
         END
         ELSE BEGIN
            SET @P_VRCUSTMEDI = 0;
         END;
      END;
      /* BRUTO */
      IF @P_NRTIPOCUST IN (8,9,10) BEGIN
         IF @V_QTDEESTO <> 0 BEGIN
            SET @P_VRCUSTMEDI = @V_VALRBRUT / @V_QTDEESTO;
         END
         ELSE BEGIN
            SET @P_VRCUSTMEDI = 0;
         END;
      END;

      IF @P_NRTIPOCUST IN (7,10) BEGIN

         DECLARE C_CDINDICEMF CURSOR FOR
         SELECT
            CDINDICEMF
         FROM
            PARAMGERAL;

         OPEN  C_CDINDICEMF;
         FETCH NEXT FROM C_CDINDICEMF INTO @V_CDINDICEMF;
         IF (@@FETCH_STATUS <> 0) BEGIN
            SET @V_CDINDICEMF = ' ';
         END;
         CLOSE C_CDINDICEMF;
         DEALLOCATE C_CDINDICEMF;

         SET @V_DTREFE = @V_DTPROCESSA;

         DECLARE C_FIND_MOVINDDI CURSOR FOR
         SELECT
            VRINDICE
         FROM
            MOVINDICE
         WHERE
            (CDTPINDICE = '1')    AND
            (DTREFIND = @V_DTREFE) AND
            (CDINDICE = @V_CDINDICEMF);

         OPEN  C_FIND_MOVINDDI;
         FETCH NEXT FROM C_FIND_MOVINDDI INTO @V_VRINDICE;
         IF (@@FETCH_STATUS <> 0) BEGIN
            SET @V_VRINDICE = 0;
         END;
         CLOSE C_FIND_MOVINDDI;
         DEALLOCATE C_FIND_MOVINDDI;

         IF @V_VRINDICE <> 0 BEGIN
            SET @P_VRCUSTMEDI = (@P_VRCUSTMEDI/@V_VRINDICE);
         END
         ELSE BEGIN
            SET @P_VRCUSTMEDI = 0;
         END;
      END;
   END;
   /* CUSTO DE COTACAO - LISTA DE PRECOS - CALCULOS IDENTICOS A LIB00500 */
   IF @P_NRTIPOCUST IN (11) BEGIN
      SELECT
         @V_CDFILCENTCUST = CDFILCENTCMP,
         @V_CDREGIAO = CDREGIAO
      FROM
         FILIAL
      WHERE
         CDFILIAL = @P_CDFILIAL;

      IF (@V_CDFILCENTCUST IS NULL) OR (@V_CDFILCENTCUST = '') BEGIN
         SET @V_CDFILCENTCUST = @P_CDFILIAL;
      END;

      SELECT @V_VRLISTAPRECO = AVG(ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV)
      FROM ITEMCOLE, PRODUTO
      WHERE
         ITEMCOLE.IDITMLISPREC = 'S'               AND
         ITEMCOLE.CDFILIAL     = @V_CDFILCENTCUST   AND
         ITEMCOLE.CDREGIAO     = @V_CDREGIAO        AND
         PRODUTO.CDPRODESTO    = @P_CDPRODUTO       AND
         ITEMCOLE.CDPRODUTO    = PRODUTO.CDPRODUTO AND
         ITEMCOLE.DTINIVALITM  = (SELECT MAX(I.DTINIVALITM)
                                  FROM   ITEMCOLE I
                                  WHERE
                                    I.IDITMLISPREC = 'S'                   AND
                                    ITEMCOLE.CDFILIAL    = @V_CDFILCENTCUST AND
                                    ITEMCOLE.CDREGIAO    = @V_CDREGIAO      AND
                                    ITEMCOLE.CDFORNECED  = I.CDFORNECED    AND
                                    ITEMCOLE.CDPRODUTO   = I.CDPRODUTO     AND
                                    ITEMCOLE.CDMARCA     = I.CDMARCA);

      IF @V_VRLISTAPRECO IS NULL BEGIN
         SET @V_VRLISTAPRECO = 0;
      END;

      SET @P_VRULTICOTA = @V_VRLISTAPRECO;

   END;

   IF @P_VRULTICUST IS NULL BEGIN
      SET @P_VRULTICUST = 0;
   END;

   IF @P_VRULTICOTA IS NULL BEGIN
      SET @P_VRULTICOTA = 0;
   END;

   IF @P_VRCUSTMEDI IS NULL BEGIN
      SET @P_VRCUSTMEDI = 0;
   END;
END;
GO
